
package entities;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;

import lombok.Data;
import lombok.EqualsAndHashCode;

@Data
// Here we rely on the id values being autogenerated!
@EqualsAndHashCode(of={"id"})
@Entity
public class Department implements Serializable {
    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private int id;

    private String name;

    // Inverse side of a many-to-one relationship
    @OneToMany(mappedBy = "department", fetch = FetchType.LAZY)
    // Order employees by their name, descending (when fetching from database)
    @OrderBy("name DESC")
    private List<Employee> employees;

    public Department() {
        employees = new LinkedList<>();
    }

    public void addEmployee(Employee e) {
        this.employees.add(e);
    }
}
