
package entities;

import java.io.Serializable;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import lombok.Data;
import lombok.EqualsAndHashCode;

@Data
//Here we rely on the id values being autogenerated!
@EqualsAndHashCode(of={"id"})
@Entity
public class Meeting implements Serializable {
    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private int id;

    private String title;

    @Temporal(TemporalType.TIMESTAMP)
    @Column(name = "start_stamp")
    private Calendar start;

    @Temporal(TemporalType.TIMESTAMP)
    @Column(name = "finish_stamp")
    private Calendar finish;

    // "Inverse side" of a many-to-many relationship
    @ManyToMany(mappedBy = "meetings")
    private List<Employee> employees = new LinkedList<>();


    public void addEmployee(Employee e) {
        this.employees.add(e);
    }
}
