package main.tcp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;

import lombok.Cleanup;

public class ClientMain {

    private static final String HOST = "localhost";
    private static final int PORT = 1234;

    public static void main(String[] args) {
        try {
            // Open connection to the server;
        	// s.close() will be invoked in the end,
        	// because we are using @Cleanup from Lombok
            @Cleanup Socket s = new Socket(HOST, PORT);

    		InputStream is = s.getInputStream();
    		InputStreamReader isr = new InputStreamReader(is);
    		BufferedReader br = new BufferedReader(isr);

			OutputStream os = s.getOutputStream();
			PrintWriter pw = new PrintWriter(os);

			sendRequest(pw);
            receiveResponse(br);
        } catch (Exception ex) {
            Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

	private static void receiveResponse(BufferedReader br) throws IOException {
		// Receive response
		String response = br.readLine();

		// Process response
		System.out.println("(client) Response: " + response);
	}

	private static void sendRequest(PrintWriter pw) throws IOException {
		// Create request
		String request = "This is a test message.";

		// Send request
		pw.println(request);
		pw.flush(); // important: immediately write contents to the socket
	}
}
