package example;

import java.util.Random;

import javax.ejb.NoSuchEJBException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import example.stateful.CalculatorException;
import example.stateful.RandomStatefulCalculator;
import example.stateful.Result;
import example.stateful.ResultStorage;
import example.stateful.StatefulCalculator;
import utils.JndiUtils;

public class Main {
	static StatefulCalculator calculator;
	static StatefulCalculator rndCalculator;
	static ResultStorage storage;

	public static void main(String[] args) throws NamingException {
		lookupBeans();

		Random rndGen = new Random();
		int num = rndGen.nextInt(1000);
		int num2 = rndGen.nextInt(1000);

		testException(num);
		testCalculation(num, num2);
		testLogging();
		testRandom();

		testRemoval();

		System.out.println("Done.");
	}

	private static void testRandom() {
		rndCalculator.add(0);
		int result = rndCalculator.getResult();
		System.out.println("Random result is " + result);
	}

	private static void testRemoval() {
		try {
			calculator.add(0);
		} catch (NoSuchEJBException e) {
			System.out.println("The bean instance is removed by calling getResult()!");
			e.printStackTrace();
		}
	}

	private static void testLogging() {
		// Print the collected results
		System.out.println("Results so far:");
		for (Result r : storage.getAllResults()) {
			System.out.println(r.getResult() + " @ " + r.getWhen());
		}
	}

	private static void testCalculation(int num, int num2) {
		// Do some calculations
		calculator.add(num + num2 + 1);
		// ...

		System.out.printf("The result should be %d%n", num2+1);

		// The instance will be removed after this call!
		int result = calculator.getResult();

		System.out.println("Total: " + result);
	}

	private static void testException(int num) {
		// Testing the exception of the sqrt method
		calculator.sub(num);
		try {
			calculator.sqrt();
		} catch (CalculatorException e) {
			System.out.println("Calculation error: " + e.getMessage());
		}
	}

	// Lookup the beans with JNDI from this standalone client.
	// (The handling of NamingExceptions is omitted for this example.)
	// The initial context setup is done by the gf-client.jar,
	// which is added to the project's build path.
	private static void lookupBeans() throws NamingException {
		Context ctx = new InitialContext();
		String calcName = JndiUtils.makeJndiName(
				"stateful-calculator-app",
				"stateful-calculator-ejb",
				"CalculatorBean",
				StatefulCalculator.class);
		String rndCalcName = JndiUtils.makeJndiName(
				"stateful-calculator-app",
				"stateful-calculator-ejb",
				"RandomCalculatorBean",
				RandomStatefulCalculator.class);
		String storageName = JndiUtils.makeJndiName(
				"stateful-calculator-app",
				"stateful-calculator-ejb",
				"ResultStorageBean",
				ResultStorage.class);
		calculator = (StatefulCalculator) ctx.lookup(calcName);
		rndCalculator = (RandomStatefulCalculator) ctx.lookup(rndCalcName);
		storage = (ResultStorage) ctx.lookup(storageName);
		ctx.close();
	}

}
