package example.stateful;

import javax.interceptor.AroundInvoke;
import javax.interceptor.Interceptor;
import javax.interceptor.InvocationContext;

// An interceptor class which will replace the return value of any business
// method named `getResult` with a random integer number.
@Interceptor
public class RandomResultGenerator {

	@AroundInvoke
	private Object invoke(InvocationContext ctx) throws Exception {
		if ("getResult".equals(ctx.getMethod().getName())) {
			// Return a random return value without invoking the method
			return (int) Math.round(Math.random() * Integer.MAX_VALUE);
		} else {
			// All other methods can run and return in the normal way
			return ctx.proceed();
		}
	}

}
