import javax.ejb.EJB;

import example.stateless.StatelessCalculator;
import example.stateless.ValueHolder;


//Running in an application client container:
// $GLASSFISH/glassfish/bin/appclient -cp .;stateless-calculator-client-interface.jar;stateless-calculator-client-dependencyInjection.jar Main
public class Main {

	// Get a reference to the bean by dependency injection provided by
	// the application client container. The mappedName is a vendor-specific
	// (non-portable) name of the component.
	@EJB(mappedName="statelessCalc")
	private static StatelessCalculator calculator;

	// A portable name can be used as well.
	@EJB(lookup="java:global/stateless-calculator-application/stateless-calculator-ejb/statelessCalcBean!example.stateless.Calculator")
	private static StatelessCalculator calculator2;

	public static void main(String[] args) {
		doSomething(calculator);
		doSomething(calculator2);
	}

	private static void doSomething(StatelessCalculator calc) {
		// Call bean methods
		ValueHolder result;

		result = calc.add(new ValueHolder(41), new ValueHolder(1));
		System.out.println("The sum of 41 and 1 is " + result.value);

		// Changing the value in the result will not have any effect in the
		// server, because it is passed by-copy.
		result.value = 2;

		// It is not sure this method will be called on the same
		// bean instance in the server as the previous.
		result = calculator.sqrt(result);
		System.out.println("The square root of 2 is " + result.value);
	}
}
