import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;


/**
 * Utilities for JNDI (Java Naming and Directory Interface).
 */
public class JndiUtils {
	/**
	 * Looks up the JNDI name {@code jndiName}, and returns the found object.
	 * In our case, it will be an EJB.
	 */
	static Object getBeanByJndi(String jndiName) {
		Context ctx = null;
		try {
			Properties environment = createEnvironment();
			
			// Find the EJB with a JNDI lookup
			ctx = new InitialContext(environment);
			return ctx.lookup(jndiName);
		} catch(NamingException ex) {
			ex.printStackTrace();
			return null;
		} finally {
			if (ctx != null) {
				try {
					ctx.close();
				} catch (NamingException e) {
					e.printStackTrace();
					return null;
				}
			}
		}
	}

	static Properties createEnvironment() {
		// Explicitly specify the location of the server
		// (these are the default values, so this block is unnecessary)
		Properties environment = new Properties();
		environment.setProperty("org.omg.CORBA.ORBInitialHost", "127.0.0.1");
		environment.setProperty("org.omg.CORBA.ORBInitialPort", "3700");
		return environment;
	}

	/**
	 * For more information on portable global JNDI names see:
	 * {@link https://blogs.oracle.com/MaheshKannan/entry/portable_global_jndi_names}
	 * @return The portable JNDI name of the bean.
	 */
	static String makeJndiName(String applicationName, String moduleName, String beanName, Class<?> interfaceClass) {
		String fullyQualifiedBeanInterfaceName = interfaceClass.getCanonicalName();
		return String.format("java:global/%s/%s/%s!%s", applicationName, moduleName, beanName, fullyQualifiedBeanInterfaceName);
	}
	
	static void listAvailableBeans() {
		Context ctx = null;
		try {
			Properties environment = createEnvironment();
			
			ctx = new InitialContext(environment);
			NamingEnumeration<NameClassPair> list = ctx.list("");
			while (list.hasMore()) {
				System.out.println(list.next().getName());
			}
		} catch(NamingException ex) {
			ex.printStackTrace();
			return;
		} finally {
			if (ctx != null) {
				try {
					ctx.close();
				} catch (NamingException e) {
					e.printStackTrace();
					return;
				}
			}
		}
	}
}
