package example.tx.servlet;

import java.io.IOException;
import java.io.PrintWriter;

import javax.ejb.EJBException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import example.tx.Messages;
import example.tx.Messages.ErrorType;

@WebServlet("/custom-tx-stateful")
public class CustomTxStatefulTest extends HttpServlet {
	private static final long serialVersionUID = 1L;

	@Override
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		Messages messages = lookupBean(response);
		if (null == messages) {
			return;
		}

		PrintWriter pw = response.getWriter();
		pw.println("<pre>");

		try {
			messages.add("None", ErrorType.None);
		} catch (Exception e) {
			printError(pw, e);
		}

		try {
			// EJB state is removed when an EJBException is thrown:
			// messages.add("Runtime", ErrorType.Runtime);
			// messages.add("Sys", ErrorType.Sys);
		} catch (Exception e) {
			printError(pw, e);
		}

		try {
			messages.add("AppNonRollback", ErrorType.AppNonRollback);
		} catch (Exception e) {
			printError(pw, e);
		}

		try {
			// When enabled, the transaction gets rolled back
			// messages.add("AppRollback", ErrorType.AppRollback);
		} catch (Exception e) {
			printError(pw, e);
		}

		printMessages(messages, pw);

		// Try to call getMessages without a transaction
		messages = lookupBean(response);
		if (null == messages) {
			return;
		}
		try {
			messages.getMessages();
		} catch (Exception e) {
			printError(pw, e);
		}
	}

	private Messages lookupBean(HttpServletResponse response)
			throws IOException {
		Context ctx = null;
		try {
			ctx = new InitialContext();
			Messages messages = (Messages) ctx
					.lookup("java:global/tx-exception-app/tx-exception-ejb/CustomTxStatefulMessagesBean");
			return messages;
		} catch (NamingException e) {
			response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
					e.getMessage());
			return null;
		} finally {
			if (null != ctx) {
				try {
					ctx.close();
				} catch (NamingException e) {
				}
			}
		}
	}

	private void printMessages(Messages messages, PrintWriter pw) {
		messages.getMessages().forEach(pw::println);
	}

	private void printError(PrintWriter pw, Throwable e) {
		String message = e.getMessage();
		if (e instanceof EJBException && null != e.getCause()) {
			message = e.getCause().getMessage();
		}
		pw.println("Exception: " + message);
	}
}
