package example.tx.servlet;

import java.io.IOException;

import javax.ejb.EJBException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import example.tx.Messages;
import example.tx.Messages.ErrorType;
import example.tx.exceptions.MessageException;

@WebServlet("/stateful")
public class StatefulTest extends HttpServlet {
	private static final long serialVersionUID = 1L;

	@Override
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		Messages bean = (Messages) request.getSession().getAttribute("bean");
		if (null != bean) {
			// List previous messages
			request.setAttribute("messages", bean.getMessages());
		}
		request.getRequestDispatcher("/WEB-INF/messages.jsp").forward(request,
				response);

		// Clear session
		request.getSession().removeAttribute("bean");
		request.getSession().removeAttribute("error");
	}

	@Override
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		String textParam = request.getParameter("text");
		String errorTypeParam = request.getParameter("errorType");
		HttpSession session = request.getSession();

		Messages bean;
		Context ctx;
		try {
			ctx = new InitialContext();
			bean = (Messages) ctx
					.lookup("java:global/tx-exception-app/tx-exception-ejb/StatefulMessagesBean");
			ctx.close();
			session.setAttribute("bean", bean);
		} catch (NamingException e) {
			response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
					e.getMessage());
			return;
		}

		if (null != textParam && !"".equals(textParam)
				&& null != errorTypeParam) {
			ErrorType errorType;
			try {
				errorType = ErrorType.valueOf(errorTypeParam);
			} catch (Exception e) {
				errorType = ErrorType.None;
			}

			try {
				bean.add(textParam, errorType);
			} catch (MessageException e) {
				session.setAttribute("error", e.getMessage());
			} catch (EJBException e) {
				String error;
				if (null != e.getCause()) {
					// Runtime exceptions are automatically wrapped
					error = e.getCause().getMessage();
				} else {
					error = e.getMessage();
				}
				session.setAttribute("error", "EJBException: " + error);
				// The bean will be unusable
				session.removeAttribute("bean");
			}
		}

		response.sendRedirect(request.getRequestURI());
	}
}
